@@ -2,7 +2,17 @@ import inspect
 import json
 import re
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Callable, Generic, List, Optional, TypeVar, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Generic,
+    List,
+    Optional,
+    Set,
+    TypeVar,
+    Union,
+)
 
 from async_lambda.util import make_cf_tags
 
@@ -13,6 +23,9 @@ from ..config import config
 if TYPE_CHECKING:
     from ..controller import AsyncLambdaController  # pragma: not covered
 
+from ..middleware import RT, MiddlewareStackExecutor
+from .events.dynamodb_event import DynamoDBEvent
+from .events.managed_sqs_batch_event import ManagedSQSBatchEvent
 from .events.managed_sqs_event import ManagedSQSEvent
 from .events.scheduled_event import ScheduledEvent
 from .events.unmanaged_sqs_event import UnmanagedSQSEvent
@@ -20,40 +33,55 @@ from .events.unmanaged_sqs_event import UnmanagedSQSEvent
 
 class TaskTriggerType(Enum):
     MANAGED_SQS = 1
-    UNMANAGED_SQS = 2
-    SCHEDULED_EVENT = 3
-    API_EVENT = 4
+    MANAGED_SQS_BATCH = 2
+    UNMANAGED_SQS = 3
+    SCHEDULED_EVENT = 4
+    API_EVENT = 5
+    DYNAMODB_EVENT = 6
+    BASE_EVENT = 7
 
 
+MANAGED_SQS_TASK_TYPES = {
+    TaskTriggerType.MANAGED_SQS,
+    TaskTriggerType.MANAGED_SQS_BATCH,
+}
+
 EventType = TypeVar(
-    "EventType", bound=Union[ManagedSQSEvent, ScheduledEvent, UnmanagedSQSEvent]
+    "EventType",
+    bound=Union[
+        ManagedSQSEvent,
+        ManagedSQSBatchEvent,
+        ScheduledEvent,
+        UnmanagedSQSEvent,
+        DynamoDBEvent,
+    ],
 )
 
 
-class AsyncLambdaTask(Generic[EventType]):
+class AsyncLambdaTask(Generic[EventType, RT]):
     controller: "AsyncLambdaController"
     task_id: str
     trigger_type: TaskTriggerType
     trigger_config: dict
 
     timeout: int
-    memory: int
+    memory: Optional[int]
     ephemeral_storage: int
     maximum_concurrency: Optional[Union[int, List[int]]]
     init_tasks: List[Union[Callable[[str], Any], Callable[[], Any]]]
     _has_run_init_tasks: bool
 
-    executable: Callable[[EventType], Any]
+    executable: Callable[[EventType], RT]
 
     def __init__(
         self,
         controller: "AsyncLambdaController",
-        executable: Callable[[EventType], Any],
+        executable: Callable[[EventType], RT],
         task_id: str,
         trigger_type: TaskTriggerType,
         trigger_config: Optional[dict] = None,
         timeout: int = 60,
-        memory: int = 128,
+        memory: Optional[int] = None,
         ephemeral_storage: int = 512,
         maximum_concurrency: Optional[Union[int, List[int]]] = None,
         init_tasks: Optional[
@@ -77,7 +105,7 @@ class AsyncLambdaTask(Generic[EventType]):
         self._has_run_init_tasks = False
 
         if (
-            self.trigger_type == TaskTriggerType.MANAGED_SQS
+            self.trigger_type in MANAGED_SQS_TASK_TYPES
             and "dlq_task_id" in self.trigger_config
         ):
             dlq_task_id = self.trigger_config["dlq_task_id"]
@@ -87,7 +115,7 @@ class AsyncLambdaTask(Generic[EventType]):
                     raise Exception(
                         f"Error setting DLQ Task ID: No task with the task_id {dlq_task_id} exists."
                     )
-                if dlq_task.trigger_type != TaskTriggerType.MANAGED_SQS:
+                if dlq_task.trigger_type not in MANAGED_SQS_TASK_TYPES:
                     raise Exception(
                         f"Error setting DLQ Task ID: Task {dlq_task_id} is not an async-task."
                     )
@@ -115,7 +143,7 @@ class AsyncLambdaTask(Generic[EventType]):
             raise ValueError("Task ID must be less than 32 characters long.")
 
     def get_lane_count(self) -> int:
-        if self.trigger_type != TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type not in MANAGED_SQS_TASK_TYPES:
             raise Exception(f"The task {self.task_id} is not a managed queue task.")
         if "lane_count" in self.trigger_config and isinstance(
             self.trigger_config["lane_count"], int
@@ -124,7 +152,7 @@ class AsyncLambdaTask(Generic[EventType]):
         return self.controller.get_lane_count()
 
     def should_propagate_lane_assignment(self) -> bool:
-        if self.trigger_type != TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type not in MANAGED_SQS_TASK_TYPES:
             raise Exception(f"The task {self.task_id} is not a managed queue task.")
         if "propagate_lane_assignment" in self.trigger_config and isinstance(
             self.trigger_config["propagate_lane_assignment"], bool
@@ -136,7 +164,7 @@ class AsyncLambdaTask(Generic[EventType]):
         """
         Returns the managed queue's name for this task.
         """
-        if self.trigger_type != TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type not in MANAGED_SQS_TASK_TYPES:
             raise Exception(f"The task {self.task_id} is not a managed queue task.")
         if lane == 0:
             return f"{config.name}-{self.task_id}"
@@ -146,12 +174,12 @@ class AsyncLambdaTask(Generic[EventType]):
         return f"{config.name}-{self.task_id}"
 
     def get_managed_queue_arn(self, lane: int = 0):
-        if self.trigger_type != TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type not in MANAGED_SQS_TASK_TYPES:
             raise Exception(f"The task {self.task_id} is not a managed queue task.")
         return f"arn:aws:sqs:{env.get_aws_region()}:{env.get_aws_account_id()}:{self.get_managed_queue_name(lane=lane)}"
 
     def get_managed_queue_url(self, lane: int = 0):
-        if self.trigger_type != TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type not in MANAGED_SQS_TASK_TYPES:
             raise Exception(f"The task {self.task_id} is not a managed queue task.")
         return f"https://sqs.{env.get_aws_region()}.amazonaws.com/{env.get_aws_account_id()}/{self.get_managed_queue_name(lane=lane)}"
 
@@ -159,21 +187,21 @@ class AsyncLambdaTask(Generic[EventType]):
         return f"{self.task_id}ALFunc"
 
     def get_managed_queue_logical_id(self, lane: int = 0):
-        if self.trigger_type != TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type not in MANAGED_SQS_TASK_TYPES:
             raise Exception(f"The task {self.task_id} is not a managed queue task.")
         if lane == 0:
             return f"{self.task_id}ALQueue"
         return f"{self.task_id}ALQueueL{lane}"
 
     def get_managed_queue_extra_logical_id(self, index: int, lane: int = 0):
-        if self.trigger_type != TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type not in MANAGED_SQS_TASK_TYPES:
             raise Exception(f"The task {self.task_id} is not a managed queue task.")
         if lane == 0:
             return f"{self.get_function_logical_id()}Extra{index}"
         return f"{self.get_function_logical_id()}Extra{index}L{lane}"
 
     def get_managed_queue_event_logical_id(self, lane: int = 0):
-        if self.trigger_type != TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type not in MANAGED_SQS_TASK_TYPES:
             raise Exception(f"The task {self.task_id} is not a managed queue task.")
         if lane == 0:
             return "ManagedSQS"
@@ -183,14 +211,14 @@ class AsyncLambdaTask(Generic[EventType]):
         sqs_properties = {}
         if (
             isinstance(self.maximum_concurrency, list)
-            and self.trigger_type != TaskTriggerType.MANAGED_SQS
+            and self.trigger_type not in MANAGED_SQS_TASK_TYPES
         ):
             raise Exception(
                 f"Invalid maximum concurrency configuration for task {self.task_id}. Must be an int, not a list of ints. Lanes are only supported for ManagedSQS tasks."
             )
         if (
             isinstance(self.maximum_concurrency, list)
-            and self.trigger_type == TaskTriggerType.MANAGED_SQS
+            and self.trigger_type in MANAGED_SQS_TASK_TYPES
             and len(self.maximum_concurrency) != self.get_lane_count()
         ):
             raise Exception(
@@ -200,7 +228,7 @@ class AsyncLambdaTask(Generic[EventType]):
             sqs_properties["ScalingConfig"] = {
                 "MaximumConcurrency": self.maximum_concurrency
             }
-        if self.trigger_type == TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type in MANAGED_SQS_TASK_TYPES:
             events = {}
             for lane_index in range(self.get_lane_count()):
                 sqs_properties = {}
@@ -212,10 +240,18 @@ class AsyncLambdaTask(Generic[EventType]):
                     sqs_properties["ScalingConfig"] = {
                         "MaximumConcurrency": self.maximum_concurrency
                     }
+                if (
+                    self.trigger_config.get("max_batching_window")
+                    or self.trigger_config["batch_size"] > 10
+                ):
+                    sqs_properties["MaximumBatchingWindowInSeconds"] = (
+                        self.trigger_config.get("max_batching_window") or 30
+                    )
+
                 events[self.get_managed_queue_event_logical_id(lane=lane_index)] = {
                     "Type": "SQS",
                     "Properties": {
-                        "BatchSize": 1,
+                        "BatchSize": self.trigger_config["batch_size"],
                         "Enabled": True,
                         "Queue": {
                             "Fn::GetAtt": [
@@ -262,10 +298,27 @@ class AsyncLambdaTask(Generic[EventType]):
                     },
                 }
             }
+        elif self.trigger_type == TaskTriggerType.DYNAMODB_EVENT:
+            return {
+                "DynamoDBEvent": {
+                    "Type": "DynamoDB",
+                    "Properties": {
+                        "Stream": self.trigger_config["stream_arn"],
+                        "StartingPosition": "TRIM_HORIZON",
+                        "BatchSize": self.trigger_config["batch_size"],
+                        "MaximumBatchingWindowInSeconds": self.trigger_config[
+                            "max_batching_window"
+                        ],
+                        "Enabled": True,
+                    },
+                }
+            }
+        elif self.trigger_type == TaskTriggerType.BASE_EVENT:
+            return {}
         raise NotImplementedError()
 
     def get_policy_sqs_resources(self) -> List[dict]:
-        if self.trigger_type == TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type in MANAGED_SQS_TASK_TYPES:
             return [
                 {
                     "Fn::GetAtt": [
@@ -279,10 +332,21 @@ class AsyncLambdaTask(Generic[EventType]):
             return [self.trigger_config["queue_arn"]]
         return []
 
+    @classmethod
+    def get_policy_external_task_resources(cls, external_task_id: str) -> list:
+        return [
+            {
+                "Fn::Sub": "arn:aws:sqs:${AWS::Region}:${AWS::AccountId}:external_task_id".replace(
+                    "external_task_id", external_task_id
+                )
+            }
+        ]
+
     def get_sam_template(
         self,
         module: str,
         tasks: List["AsyncLambdaTask"],
+        external_async_tasks: Set[str],
         config_dict: dict,
         stage: Optional[str] = None,
     ) -> dict:
@@ -313,8 +377,14 @@ class AsyncLambdaTask(Generic[EventType]):
         managed_tasks_resources = [
             resource
             for task in tasks
-            if task.trigger_type == TaskTriggerType.MANAGED_SQS
+            if task.trigger_type in MANAGED_SQS_TASK_TYPES
             for resource in task.get_policy_sqs_resources()
+        ] + [
+            resource
+            for external_async_task_id in external_async_tasks
+            for resource in self.get_policy_external_task_resources(
+                external_async_task_id
+            )
         ]
         if len(managed_tasks_resources) > 0:
             policy_statements.append(
@@ -354,6 +424,8 @@ class AsyncLambdaTask(Generic[EventType]):
                 function_properties["VpcConfig"]["SubnetIds"] = sorted(
                     build_config.subnet_ids
                 )
+        if len(build_config.logging_config) > 0:
+            function_properties["LoggingConfig"] = build_config.logging_config
 
         template = {
             self.get_function_logical_id(): {
@@ -375,7 +447,9 @@ class AsyncLambdaTask(Generic[EventType]):
                     "FunctionName": self.get_function_name(),
                     "CodeUri": ".async_lambda/build/deployment.zip",
                     "EphemeralStorage": {"Size": self.ephemeral_storage},
-                    "MemorySize": self.memory,
+                    "MemorySize": (
+                        self.memory if self.memory else config.default_task_memory
+                    ),
                     "Timeout": self.timeout,
                     "Events": events,
                     "Policies": [
@@ -387,7 +461,7 @@ class AsyncLambdaTask(Generic[EventType]):
             }
         }
 
-        if self.trigger_type == TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type in MANAGED_SQS_TASK_TYPES:
             dlq_task = self.get_dlq_task()
             if dlq_task is None:
                 dead_letter_target_arn = {
@@ -408,10 +482,7 @@ class AsyncLambdaTask(Generic[EventType]):
                     "async-lambda-lane": str(lane_index),
                     "async-lambda-queue-type": "managed",
                 }
-                if (
-                    self.trigger_type == TaskTriggerType.MANAGED_SQS
-                    and self.trigger_config["is_dlq_task"]
-                ):
+                if self.trigger_config["is_dlq_task"]:
                     _extra_tags["async-lambda-queue-type"] = "dlq-task"
                 template[self.get_managed_queue_logical_id(lane=lane_index)] = {
                     "Type": "AWS::SQS::Queue",
@@ -453,7 +524,7 @@ class AsyncLambdaTask(Generic[EventType]):
         return json.loads(stringified_extra)
 
     def get_dlq_task(self) -> Optional["AsyncLambdaTask"]:
-        if self.trigger_type != TaskTriggerType.MANAGED_SQS:
+        if self.trigger_type not in MANAGED_SQS_TASK_TYPES:
             return None
         if self.trigger_config.get("is_dlq_task"):
             return None
@@ -461,9 +532,13 @@ class AsyncLambdaTask(Generic[EventType]):
             return self.controller.get_task(self.trigger_config["dlq_task_id"])
         return self.controller.get_dlq_task()
 
-    def execute(self, event: EventType) -> Any:
+    def execute(self, event: EventType) -> RT:
         """
         Executes the tasks function
         """
         self._run_init_tasks()
-        return self.executable(event)
+        middleware = self.controller.get_middleware_for_event(event)
+        middleware_stack_executor = MiddlewareStackExecutor[EventType, RT](
+            middleware=middleware, final=self.executable
+        )
+        return middleware_stack_executor.call_next(event)
